/*******************************************************************************
 * DERMI: A DHT-based Peer-to-Peer Middleware for Distributed Components.
 * Copyright (C) 2003-2004 Carles Pairot
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ******************************************************************************/
package dermi.samples.anycall;

import dermi.*;
import dermi.exception.*;
import dermi.util.Bus;
import dermi.registry.FileRegistry;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Properties;
import java.io.IOException;

/**
 * AUTOMATICALLY GENERATED by DERMIC TOOL
 * This class implements the skeleton class for Seti object
 * @author Carles Pairot   <cpairot@etse.urv.es>
 */
public abstract class SetiSkel extends EventServer implements RemoteAnycallEventListener, Seti {
  String method;
  Hashtable msg = new Hashtable();
  Hashtable subs = new Hashtable();
  private Hashtable receivedEvents = new Hashtable();

  /**
   * Method invoked each time the skeleton receives an event
   * @param evt Hashtable Information about the received event
   */
  public SetiSkel (Properties env) throws RemoteException {
    super (env);
    className = this.getClass().getSuperclass().getName().replaceAll ("Skel", "");
    uid = dermi.util.UID.getUID();

    // If this is a replica, the UID will come from the environment properties
    if (env.getProperty (Context.CLASS_NAME).equals (className + "Stub")) {
      if (env.getProperty (Context.CLASS_ID) != null) {
        uid = env.getProperty (Context.CLASS_ID);
      }
    }

    subs.put ("OBJECT", uid);
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Method invoked each time the skeleton receives an anycall event
   * @param evt Hashtable Information about the received event
   */
  public Hashtable anycallEventArrived (Hashtable evt) throws UnmatchedAnycallMethodException {
    method = (String) evt.get ("METHOD");
    try {
      if (method.equals ("anyGetDataUnit")) {
        Vector args = (Vector) evt.get ("object");
        Object res = null;
        Hashtable oldEvent = (Hashtable) receivedEvents.get (method + (String) evt.get ("TIMESTAMP"));
        if (oldEvent != null) {
          return oldEvent;
        }
        else {
          // Check for local condition
          if (anyGetDataUnitCondition ((String) args.elementAt (0))) {
            res = anyGetDataUnit ((String) args.elementAt (0));
            Hashtable msg = new Hashtable();
            msg.put ("OBJ", uid + ":RESULT");
            msg.put ("TYPE", "DISPATCH");
            msg.put ("METHOD", "anyGetDataUnit");
            msg.put ("object", res);
            msg.put ("ANYCALL_RESULT", new Boolean (false));
            msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
            msg.put ("SOURCE_NHANDLE", evt.get ("SOURCE_NHANDLE"));
            receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), msg);
            return (msg);
          }
          else {
            // Local condition not met, message will be routed to another member of the group
            Hashtable msg = new Hashtable();
            msg.put ("OBJ", uid + ":RESULT");
            msg.put ("TYPE", "DISPATCH");
            msg.put ("METHOD", "anyGetDataUnit");
            msg.put ("object", evt.get ("object"));
            msg.put ("RESULT", "VOID");
            msg.put ("ANYCALL_RESULT", new Boolean (true));
            msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
            msg.put ("SOURCE_NHANDLE", evt.get ("SOURCE_NHANDLE"));
            receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), msg);
            return (msg);
          }
        }
      }
    } catch(RemoteException ex) {
      Hashtable msg = new Hashtable();
      msg.put ("OBJ", uid + ":RESULT");
      msg.put ("TYPE", "EXCEPTION");
      msg.put ("METHOD", "anyGetDataUnit");
      msg.put ("object", ex);
      msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
      msg.put ("SOURCE_NHANDLE", evt.get ("SOURCE_NHANDLE"));
      receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), msg);
      return msg;
    }
    throw new UnmatchedAnycallMethodException (method);
  }

  /**
   * Method invoked each time the skeleton receives an event
   * @param evt Hashtable Information about the received event
   */
  public void eventArrived (Hashtable evt) {
    method = (String)evt.get ("METHOD");
    if (method.equals ("getBytes")) {
      try {
        String aClass = Bus.getClassName (className);
        String fileClass = classStore + aClass + "Stub.class";
        byte data[] = getBytes (fileClass);
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULTDYNAMIC");
        msg.put ("TYPE", "DISPATCH");
        msg.put ("METHOD", "getBytes");
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        dispatchEvent (msg, data);
      } catch (IOException ex1) {
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULTDYNAMIC");
        msg.put ("TYPE", "EXCEPTION");
        msg.put ("METHOD", "getBytes");
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        try {
          dispatchEvent (msg, ex1);
        } catch (RemoteException e2) {
          e2.printStackTrace();
        }
      } catch (RemoteException ex2) {
        ex2.printStackTrace();
      }
    }
  }

  /**
   * Closes the skeleton
   * @throws RemoteException If something goes wrong ;-)
   */
  public void close() throws RemoteException {
    try {
      conn.unsubscribe (this, subs);
      dermi.util.Bus.close();
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Initializes the skeleton
   * @throws RemoteException If something goes wrong ;-)
   */
  public void init() throws RemoteException {
    conn = dermi.util.Bus.getBus (eref);
    try {
      conn.subscribe (this, subs);
    }catch (SessionException e) {
      throw new ConnectionException(e.getMessage());
    }
  }

  /**
   * Returns a stub copy of this class
   * @throws RemoteException If something goes wrong ;-)
   * @return ERemote The returned stub
   */
  public ERemote copy() throws RemoteException {
    ERemote ref = FileRegistry.loadStub (getERef());
    return (ERemote) ref;
  }
}
