/*******************************************************************************
 * DERMI: A DHT-based Peer-to-Peer Middleware for Distributed Components.
 * Copyright (C) 2003-2004 Carles Pairot
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ******************************************************************************/
package dermi.samples.directcall;

import dermi.*;
import dermi.exception.*;
import dermi.registry.FileRegistry;

import java.util.*;
import java.io.*;

/**
 * AUTOMATICALLY GENERATED by DERMIC TOOL
 * This class implements the stub class for Simple object
 * @author Carles Pairot   <cpairot@etse.urv.es>
 */
public class SimpleStub extends EventServer implements RemoteEventListener, Simple, ERemote {
  private Hashtable subs = new Hashtable();
  private boolean mergeResultArrived = false;
  private boolean setAgeResultArrived = false;
  private boolean getAgeResultArrived = false;
  private boolean exception = false;
  private Object mergeResult;
  private Object setAgeResult;
  private Object getAgeResult;
  private String method;
  private int timeout;
  private HashSet receivedEvents = new HashSet();
  private rice.p2p.commonapi.NodeHandle serverNh;

  /**
   * Constructor: entry point to the stub class
   * @param env Properties Environment properties
   * @throws RemoteException If something goes wrong ;-)
   */
  public SimpleStub (Properties env) throws RemoteException {
    super (env);
    String muid = env.getProperty (Context.CLASS_ID);
    uid = muid;
    className = this.getClass().getName().replaceAll ("Stub", "");
    subs.put ("OBJ", uid + ":RESULT");

    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }

    try {
      FileInputStream fos = new FileInputStream ("nodehandle.data");
      ObjectInputStream bo = new ObjectInputStream (fos);
      serverNh = (rice.p2p.commonapi.NodeHandle) bo.readObject();
      ((rice.pastry.NodeHandle) serverNh).setLocalNode ((rice.pastry.PastryNode) conn.getLocalNode());
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Constructor: entry point to the stub class
   * @param env Properties Environment properties
   * @param stub String Stub name
   * @throws RemoteException  If something goes wrong ;-)
   */
  public SimpleStub (Properties env, String stub) throws RemoteException {
    super (env);
    String muid = env.getProperty (Context.CLASS_ID);
    uid = muid;
    className = this.getClass().getName().replaceAll ("Stub", "");
    subs.put ("OBJ", uid + ":RESULT");
  }

  /**
   * Method invoked each time the stub received an event
   * @param evt Hashtable Information about the received event
   */
  public void eventArrived (Hashtable evt) {
    method = (String) evt.get ("METHOD");
    if (!receivedEvents.contains (method + (String) evt.get ("TIMESTAMP"))) {
      String type;
      if (method.equals ("merge")) {
        mergeResult = evt.get ("object");
        type = (String) evt.get ("TYPE");
        if (type.equals ("EXCEPTION"))
          exception = true;
        mergeResultArrived = true;
      }
      else if (method.equals ("setAge")) {
        setAgeResult = evt.get ("object");
        type = (String) evt.get ("TYPE");
        if (type.equals ("EXCEPTION")) {
          exception = true;
        }
        setAgeResultArrived = true;
      }
      else if (method.equals ("getAge")) {
        getAgeResult = evt.get ("object");
        type = (String) evt.get ("TYPE");
        if (type.equals ("EXCEPTION"))
          exception = true;
        getAgeResultArrived = true;
      }
      receivedEvents.add (method + (String) evt.get ("TIMESTAMP"));
    }
  }

  /**
   * Generated stub method: merge
   * @param p0 Integer Input parameter 0
   * @param p1 String Input parameter 1
   * @param p2 Integer Input parameter 2
   * @throws RemoteException  If something goes wrong ;-)
   * @return String Return result
   */
  public String merge (Integer p0, String p1, Integer p2) throws RemoteException {
    // Set timestamp for this event and its possible retransmissions
    String tstamp = "" + System.currentTimeMillis();

    for (int retransmission = 0; retransmission < Context.RETRANSMISSIONS;) {
      Hashtable evt = new Hashtable();
      evt.put ("OBJECT", uid);
      evt.put ("TYPE", "DISPATCH");
      evt.put ("METHOD", "merge");
      evt.put ("TIMESTAMP", tstamp);
      evt.put ("SOURCE_NHANDLE", conn.getLocalNodeHandle());
      evt.put ("DEST_NHANDLE", serverNh);
      mergeResultArrived = false;
      Vector args = new Vector();
      args.add (p0);
      args.add (p1);
      args.add (p2);
      dispatchDirectEvent (evt, args);
      timeout = 0;

      while (!mergeResultArrived && timeout < Context.TIMEOUT) {
        try {
          Thread.currentThread().sleep (Context.DELAY);
        } catch (InterruptedException ex) {
          ex.printStackTrace();
        }
        timeout++;
      }
      if (exception) {
        exception = false;
        RemoteException ex = (RemoteException) mergeResult;
        throw ex;
      }
      else if (timeout == Context.TIMEOUT) {
        retransmission++;
        continue;
      }
      break;
    }
    if (timeout == Context.TIMEOUT) {
      throw new TimeoutException ("Object seems to be down.");
    }
    else {
      String res = (String) mergeResult;
      return res;
    }
  }

  /**
   * Generated stub method: setAge
   * @param p0 String Input parameter 0
   * @throws RemoteException  If something goes wrong ;-)
   */
  public void setAge (String p0) throws RemoteException {
    // Set timestamp for this event and its possible retransmissions
    String tstamp = "" + System.currentTimeMillis();

    for (int retransmission = 0; retransmission < Context.RETRANSMISSIONS;) {
      Hashtable evt = new Hashtable();
      evt.put ("OBJECT", uid);
      evt.put ("TYPE", "DISPATCH");
      evt.put ("METHOD", "setAge");
      evt.put ("TIMESTAMP", tstamp);
      evt.put ("SOURCE_NHANDLE", conn.getLocalNodeHandle());
      evt.put ("DEST_NHANDLE", serverNh);
      setAgeResultArrived = false;
      Vector args = new Vector();
      args.add (p0);
      dispatchDirectEvent (evt, args);
      timeout = 0;
      while (!setAgeResultArrived && timeout < Context.TIMEOUT) {
        try {
          Thread.currentThread().sleep (Context.DELAY);
        }
        catch (InterruptedException ex) {
          ex.printStackTrace();
        }
        timeout++;
      }
      if (exception) {
        exception = false;
        RemoteException ex = (RemoteException) setAgeResult;
        throw ex;
      }
      else if (timeout == Context.TIMEOUT) {
        retransmission++;
        continue;
      }
      break;
    }
    if (timeout == Context.TIMEOUT) {
      throw new TimeoutException ("Object seems to be down.");
    }
  }

  /**
   * Generated stub method: getAge
   * @throws RemoteException  If something goes wrong ;-)
   * @return String Return result
   */
  public String getAge() throws RemoteException {
    // Set timestamp for this event and its possible retransmissions
    String tstamp = "" + System.currentTimeMillis();

    for (int retransmission = 0; retransmission < Context.RETRANSMISSIONS;) {
      Hashtable evt = new Hashtable();
      evt.put ("OBJECT", uid);
      evt.put ("TYPE", "DISPATCH");
      evt.put ("METHOD", "getAge");
      evt.put ("TIMESTAMP", tstamp);
      evt.put ("SOURCE_NHANDLE", conn.getLocalNodeHandle());
      evt.put ("DEST_NHANDLE", serverNh);
      getAgeResultArrived = false;
      dispatchDirectEvent (evt);
      timeout = 0;
      while (!getAgeResultArrived && timeout < Context.TIMEOUT) {
        try {
          Thread.currentThread().sleep (Context.DELAY);
        } catch (InterruptedException ex) {
          ex.printStackTrace();
        }
        timeout++;
      }
      if (exception) {
        exception = false;
        RemoteException ex = (RemoteException) getAgeResult;
        throw ex;
      }
      else if (timeout == Context.TIMEOUT) {
        retransmission++;
        continue;
      }
      break;
    }
    if (timeout == Context.TIMEOUT) {
      throw new TimeoutException ("Object seems to be down.");
    }
    else {
      String res = (String) getAgeResult;
      return res;
    }
  }

  /**
   * Closes the stub
   * @throws RemoteException  If something goes wrong ;-)
   */
  public void close() throws RemoteException {
    try {
      conn.unsubscribe (this, subs);
      dermi.util.Bus.close();
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Initializes the stub
   * @throws RemoteException  If something goes wrong ;-)
   */
  public void init() throws RemoteException {
    conn = dermi.util.Bus.getBus (eref);
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Writes the current object to a stream
   * @param out ObjectOutputStream Data stream
   * @throws IOException  If something goes wrong ;-)
   */
  private void writeObject (ObjectOutputStream out) throws IOException {
    out.defaultWriteObject();
  }

  /**
   * Reads an object from a stream
   * @param in ObjectInputStream Data stream
   * @throws IOException  If something goes wrong ;-)
   * @throws ClassNotFoundException  If something goes wrong ;-)
   */
  private void readObject (ObjectInputStream in) throws IOException, ClassNotFoundException {
    try {
      in.defaultReadObject();
      init();
    } catch (RemoteException e) {
      e.printStackTrace();
    }
  }

  /**
   * Compares this object with another
   * @param obj Object The other object
   * @return boolean true if both equal
   */
  public boolean equals (Object obj) {
    ERemote ref = (ERemote) obj;
    return ref.getERef().equals (getERef());
  }

  /**
   * Returns a stub copy of this class
   * @throws RemoteException If something goes wrong ;-)
   * @return ERemote The returned stub
   */
  public ERemote copy() throws RemoteException {
    ERemote ref = FileRegistry.loadStub (getERef());
    return (ERemote) ref;
  }
}
