/*******************************************************************************
 * DERMI: A DHT-based Peer-to-Peer Middleware for Distributed Components.
 * Copyright (C) 2003-2004 Carles Pairot
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ******************************************************************************/
package dermi.samples.reference;

import dermi.*;
import dermi.exception.*;
import dermi.util.Bus;
import dermi.registry.FileRegistry;

import rice.p2p.commonapi.NodeHandle;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Properties;
import java.io.IOException;

/**
 * AUTOMATICALLY GENERATED by DERMIC TOOL
 * This class implementes the skeleton class for Place object
 * @author Carles Pairot   <cpairot@etse.urv.es>
 */
public abstract class PlaceSkel extends EventServer implements RemoteEventListener, Place {
  String method;
  Hashtable msg = new Hashtable();
  Hashtable subs = new Hashtable();
  private Hashtable receivedEvents = new Hashtable();

  /**
   * Constructor: Entry point to the skeleton class
   * @param env Properties
   * @throws RemoteException
   */
  public PlaceSkel (Properties env) throws RemoteException {
    super (env);
    className = this.getClass().getSuperclass().getName().replaceAll ("Skel", "");
    uid = dermi.util.UID.getUID();

    // If this is a replica, the UID will come from the environment properties
    if (env.getProperty (Context.CLASS_NAME).equals (className + "Stub")) {
      if (env.getProperty (Context.CLASS_ID) != null) {
        uid = env.getProperty (Context.CLASS_ID);
      }
    }

    subs.put ("OBJECT", uid);
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Method invoked each time the skeleton receives an event
   * @param evt Hashtable Information about the received event
   */
  public void eventArrived (Hashtable evt) {
    method = (String) evt.get ("METHOD");
    if (method.equals ("join")) {
      try {
        Vector args = (Vector) evt.get ("object");

        if (receivedEvents.get (method + (String) evt.get ("TIMESTAMP")) == null) {
          join ((dermi.samples.reference.User) args.elementAt (0));
          receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), new Integer(-1));
        }

        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULT");
        msg.put ("TYPE", "DISPATCH");
        msg.put ("METHOD", "join");
        msg.put ("RESULT", "VOID");
        msg.put ("SOURCE_NHANDLE", (NodeHandle) evt.get ("SOURCE_NHANDLE"));
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        try {
          // Event is dispatched using the event service (response is sent to MANY objects)
          if (evt.get ("SOURCE_NHANDLE") == null) {
            dispatchEvent (msg);
          }
          else {
            // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
            dispatchDirectEvent (msg);
          }
        } catch (RemoteException e) {
          e.printStackTrace();
        }
      } catch (RemoteException ex) {
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULT");
        msg.put ("TYPE", "EXCEPTION");
        msg.put ("METHOD", "join");
        msg.put ("SOURCE_NHANDLE", (NodeHandle) evt.get ("SOURCE_NHANDLE"));
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), ex);
        try {
          // Event is dispatched using the event service (response is sent to MANY objects)
          if (evt.get ("SOURCE_NHANDLE") == null) {
            dispatchEvent (msg, ex);
          }
          else {
            // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
            dispatchDirectEvent (msg, ex);
          }
        } catch (RemoteException e2) {
          e2.printStackTrace();
        }
      }
    }
    else if (method.equals ("leave")) {
      try {
        Vector args = (Vector) evt.get ("object");

        if (receivedEvents.get (method + (String) evt.get ("TIMESTAMP")) == null) {
          leave ((dermi.samples.reference.User) args.elementAt (0));
          receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), new Integer(-1));
        }

        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULT");
        msg.put ("TYPE", "DISPATCH");
        msg.put ("METHOD", "leave");
        msg.put ("RESULT", "VOID");
        msg.put ("SOURCE_NHANDLE", (NodeHandle) evt.get ("SOURCE_NHANDLE"));
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        try {
          // Event is dispatched using the event service (response is sent to MANY objects)
          if (evt.get ("SOURCE_NHANDLE") == null) {
            dispatchEvent (msg);
          }
          else {
            // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
            dispatchDirectEvent (msg);
          }
        } catch (RemoteException e) {
          e.printStackTrace();
        }
      } catch (RemoteException ex) {
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULT");
        msg.put ("TYPE", "EXCEPTION");
        msg.put ("SOURCE_NHANDLE", (NodeHandle) evt.get ("SOURCE_NHANDLE"));
        msg.put ("METHOD", "leave");
        receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), ex);
        try {
          // Event is dispatched using the event service (response is sent to MANY objects)
          if (evt.get ("SOURCE_NHANDLE") == null) {
            dispatchEvent (msg, ex);
          }
          else {
            // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
            dispatchDirectEvent (msg, ex);
          }
        }catch(RemoteException e2){e2.printStackTrace();}
      }
    }
    else if (method.equals ("getUsers")) {
      Vector args = (Vector) evt.get ("object");
      Vector res = null;
      try {
        if (receivedEvents.get (method + (String) evt.get ("TIMESTAMP")) != null) {
          res = (Vector) receivedEvents.get (method + (String) evt.get ("TIMESTAMP"));
        }
        else {
          res = getUsers();
          receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), res);
        }
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULT");
        msg.put ("TYPE", "DISPATCH");
        msg.put ("METHOD", "getUsers");
        msg.put ("SOURCE_NHANDLE", (NodeHandle) evt.get ("SOURCE_NHANDLE"));
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        try {
          // Event is dispatched using the event service (response is sent to MANY objects)
          if (evt.get ("SOURCE_NHANDLE") == null) {
            dispatchEvent (msg, res);
          }
          else {
            // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
            dispatchDirectEvent (msg, res);
          }
        } catch (RemoteException e2) {
          e2.printStackTrace();
        }
      } catch (RemoteException ex) {
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULT");
        msg.put ("TYPE", "EXCEPTION");
        msg.put ("METHOD", "getUsers");
        msg.put ("SOURCE_NHANDLE", (NodeHandle) evt.get ("SOURCE_NHANDLE"));
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        receivedEvents.put (method + ((String) evt.get ("TIMESTAMP")), ex);
        try {
          // Event is dispatched using the event service (response is sent to MANY objects)
          if (evt.get ("SOURCE_NHANDLE") == null) {
            dispatchEvent (msg, ex);
          }
          else {
            // Event is dispatched routing a direct message to stub source (response is sent to ONE object)
            dispatchDirectEvent (msg, ex);
          }
        } catch (RemoteException e2) {
          e2.printStackTrace();
        }
      }
    }
    else if (method.equals("getBytes")) {
      try {
        String aClass = Bus.getClassName (className);
        String fileClass = classStore + aClass + "Stub.class";
        byte data[] = getBytes (fileClass);
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULTDYNAMIC");
        msg.put ("TYPE", "DISPATCH");
        msg.put ("METHOD", "getBytes");
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        dispatchEvent (msg, data);
      } catch (IOException ex1) {
        Hashtable msg = new Hashtable();
        msg.put ("OBJ", uid + ":RESULTDYNAMIC");
        msg.put ("TYPE", "EXCEPTION");
        msg.put ("METHOD", "getBytes");
        msg.put ("TIMESTAMP", (String) evt.get ("TIMESTAMP"));
        try {
          dispatchEvent (msg, ex1);
        } catch (RemoteException e2) {
          e2.printStackTrace();
        }
      } catch (RemoteException ex2) {
        ex2.printStackTrace();
      }
    }
  }

  /**
   * Closes the skeleton
   * @throws RemoteException If something goes wrong ;-)
   */
  public void close() throws RemoteException {
    try {
      conn.unsubscribe (this, subs);
      dermi.util.Bus.close();
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Initializes the skeleton
   * @throws RemoteException If something goes wrong ;-)
   */
  public void init() throws RemoteException {
    conn = dermi.util.Bus.getBus (eref);
    try {
      conn.subscribe (this, subs);
    }catch (SessionException e) {
      throw new ConnectionException(e.getMessage());
    }
  }

  /**
   * Returns a stub copy of this class
   * @throws RemoteException If something goes wrong ;-)
   * @return ERemote The returned stub
   */
  public ERemote copy() throws RemoteException {
    ERemote ref = FileRegistry.loadStub (getERef());
    return (ERemote) ref;
  }
}
